
## prepare the city data
##
## run at the R prompt source(file = "prepare.R", echo = TRUE)
##
## (C) ceeboo 2009

## read data into R
d <- read.csv(file="Arr_Bed_mkts_95-07.csv", header=T, na.strings="#NULL!")

names(d)

## drop full names and garbage
markets <- d$Market
d <- d[-c(4,7)]

## rename column headers
names(d) <- c("Type","City","Market","Year","Count") 

table(d$Year)

## define
levels(d$Market)
mc <- c("AU", "AT", "BE", "CA", "FI", "FR", "DE", "GR", "IT", "JP", "NL", "NO", "ES", "SE", "CH", "UK", "US")

markets <- unique(data.frame(Markets = markets, Code = d$Market))
markets <- markets[match(mc, markets$Code), ]
rownames(markets) <- seq_len(nrow(markets))
markets

# check
mc %in% levels(d$Market)

## note Zurich is missing
cities <- c("Amsterdam", "Berlin", "Brussels", "Budapest", "Helsinki", "Lisbon", "London", "Madrid", "Oslo", "Paris", "Prague", "Rome", "Stockholm", "Vienna", "Zurich", "Zagreb")

levels(d$City)
cc <- c("AMS", "BER", "BRU", "BUD", "HEL", "LIS", "LON", "MAD", "OSL", "PAR","PRG", "ROM", "STO", "VIE", "ZUR", "ZAG")

cities <- data.frame(Cities = cities, Code = cc)
cities

# check
cc %in% levels(d$City)

# types (data definitions)
tc <- c("NA","NAS","NG","NGS","NZS")

tc %in% levels(d$Type)

## reduce
d <- d[d$Type %in% tc & d$City %in% cc & d$Market %in% mc, ]
d$Type <- factor(d$Type)


# summarize availability
t <- table(d$City, d$Type)
t <- cbind(t, Total = rowSums(t))
t <- rbind(t, Total = colSums(t))
names(dimnames(t)) <- names(d)[c(2,1)]
t

# refactorize to get rid of levels which no longer exist
d$City   <- factor(d$City)
d$Market <- factor(d$Market)
d$Year   <- factor(d$Year)
levels(d$Year)

# number of data points we expect per city and market
length(mc) * length(levels(d$Year))

## make array data
da <- array(NA_integer_, c(5, 15, 17, 13), dimnames = list(Type = levels(d$Type), City = levels(d$City), Market = levels(d$Market), Year = levels(d$Year)))


for (i in seq_len(nrow(d))) {
    r <- d[i, ]
    da[r$Type, r$City, r$Market, r$Year] <- r$Count
}

# check Rome
da["NA","ROM",,]
da["NG","ROM",,]

## This suggestst the definitions were the same up to 2002.
## Therefore, we settle for NG and complete pre 2003 missing
## values with NA values.

## for each city select the type with the maximum available data
## extend array for later insertion of Zurich data
das <- array(NA_integer_, dim(da)[-1] + c(1,0,0), dimnames = c(list(City = c(dimnames(da)[[2]], "ZUR")), dimnames(da)[-c(1,2)]))

s <- c(AMS="NG", BER="NA", BRU="NA", BUD="NA", HEL="NA", LIS="NG", LON="NZS", MAD="NAS", OSL="NA", PAR="NG", PRG="NA", ROM="NG", STO="NA", VIE="NAS", ZAG="NA")

for (n in names(s))
    das[n,,] <- da[s[n],n,,] 

## insert Zurich data
load(file = "zurich_NA_95-07.rda")

for (n in rownames(zurich_NA))
    das["ZUR",n,] <- zurich_NA[n,]

## extend
n <- sum(!is.na(das["ZUR",,]))
t <- t[-nrow(t),]
t <- rbind(t, ZUR = c(n, 0, 0, 0, 0, n))
t <- rbind(t, Total = colSums(t))
t

## check
das["ZUR",,]

## now complete missing values for Rome in 1995 - 1997
das["ROM",,1:3] <- da["NA","ROM",,1:3]

# check the result
das["ROM",,]

# save data array
assign("city_ALL", das)
save(city_ALL, cities, markets, file = "city_ALL_95-07.rda", compress = TRUE)

## compute missing value statistics
t <- apply(das, 3, function(x) apply(x, 1, function(x) sum(is.na(x))))
t <- cbind(t, Total = rowSums(t))
t <- rbind(t, Total = colSums(t))
names(dimnames(t)) <- names(dimnames(das))[-2]
t

## compute runs statistics (number of consecutively
## missing observations).
na.runs <- function(x) {
    r <- integer(0)
    n <- 0
    for (x in is.na(x)) {
        if (x)
            n <- n + 1
        else if (n > 0) {
            r <- c(r, n)
            n <- 0
        }
    }
    if (n > 0)
        r <- c(r, n)
    r
}


k <- as.character(1995:2006)
t <- apply(das[,,k], 1, function(x)
     table(factor(unlist(apply(x, 1, na.runs)),
	 levels = seq(length(k)))))
t <- t(t)
t <- cbind(t, Total = as.vector(t %*% seq(length(k))))
# use R's recycling mechanism
t <- rbind(t, Total = colSums(t) * seq(length(k)))
names(dimnames(t)) <- c("City", "Runs")
t

## compute "median" market shares
n <- apply(das[,,k], 1, function(x) sum(is.na(x))) / prod(dim(das)[-1])
n
t <- apply(das[,,k], c(1,2), median, na.rm = TRUE)
t <- rowSums(t, na.rm = TRUE)
s <- t / sum(t)
t <- data.frame(Bednights = round(t), Share = round(s * 100, digits = 2),
    PM = round(n * 100, digits = 0))
t

## by marktes
t <- apply(das[,,k], c(1,2), median, na.rm = TRUE)
s <- rowSums(t, na.rm = TRUE)
data.frame(round(t / s * 100), DI =
    round(apply(t / s, 1, function(x)
	 -sum(x*log(x), na.rm = TRUE)), digits = 2))

###
